from rest_framework.permissions import BasePermission
from msb_auth.users import TokenUser
from .constants import AUTH_TOKEN_ACCESS_TYPE_ISR

class MsbPermisson(BasePermission):

	def has_permission(self, request, view):
		return True

	def has_object_permission(self, request, view, obj):
		return True


class LoginRequiredPermission(MsbPermisson):

	def has_permission(self, request, view):
		return bool(request.user and request.user.is_authenticated)


class AdminUserPermission(MsbPermisson):
	def has_permission(self, request, view):
		return bool(request.user and request.user.is_staff)


class IntraServiceRequest(LoginRequiredPermission):

	def has_permission(self, request, view):
		_user: TokenUser = request.user
		if _user.access_type == AUTH_TOKEN_ACCESS_TYPE_ISR:
			return True
		return False




from .api_view import (
	ApiView, permissions, api_details, require_permission, require_authentication,
	TokenUser, SessionData, ApiResponse, RequestInfo, RequestHeaders,
)
from .api_viewset import (ApiViewset, RestResponse)
from .constants import *
from .errors import ErrorViews
from msb_validation.decorators import (management_api, validate_request_data)

__all__ = [
	"ApiView", "ApiViewset", "api_details",
	"management_api", "validate_request_data", "permissions", "require_permission", "require_authentication",
	"TokenUser", "SessionData", "ApiResponse", "RequestInfo", "RequestHeaders", "RestResponse"
]



from django.utils.translation import gettext_lazy as _
from rest_framework.fields import CharField
from rest_framework_simplejwt.serializers import (TokenObtainPairSerializer, TokenVerifySerializer)

from .constants import (TokenFieldNames as _tfn, AUTH_REQUEST_TYPE_FIELD_NAME, AUTH_REQUEST_USER_FIELD_NAME)
from .users import TokenUser
from msb_cipher import Cipher


class MsbJwtTokenObtainSerializer(TokenObtainPairSerializer):
	"""Customizes JWT default Serializer to add more information about user"""
	username_field = AUTH_REQUEST_USER_FIELD_NAME
	default_error_messages = {
		'no_active_account': _('User Authentication Failed.')
	}

	def __init__(self, *args, **kwargs):
		super(MsbJwtTokenObtainSerializer, self).__init__(*args, **kwargs)
		self.fields[AUTH_REQUEST_TYPE_FIELD_NAME] = CharField(required=True)
		self.fields[AUTH_REQUEST_USER_FIELD_NAME] = CharField(required=True)

	@classmethod
	def format_token_data(cls, token_data, user: TokenUser):
		return token_data

	@classmethod
	def get_token(cls, user: TokenUser = None):
		try:
			token_data = super().get_token(user)
			token_data[_tfn.ID] = user.token.get(_tfn.USER_ID)
			token_data[_tfn.IP] = user.token.get(_tfn.IP)
			token_data[_tfn.IS_VALID] = user.token.get(_tfn.IS_VALID)
			token_data[_tfn.SESSION] = user.token.get(_tfn.SESSION).to_json()
			token_data[_tfn.OWNER] = Cipher.encrypt(f"{user.token.get(_tfn.OWNER)}@{user.token.get(_tfn.IP)}")
			token_data[_tfn.ACCESS_TYPE] = Cipher.encrypt(user.token.get(_tfn.ACCESS_TYPE))

			return cls.format_token_data(token_data, user)
		except Exception as e:
			return None


class MsbJwtTokenVerifySerializer(TokenVerifySerializer):
	pass



from .token import AuthToken
from django.utils.functional import cached_property


class TokenUser(AuthToken):

	@cached_property
	def username(self):
		return self.session.get('username', None)

	@property
	def is_authenticated(self):
		return self.session.get('is_authenticated', False) and self.is_valid

	@cached_property
	def is_admin(self):
		return self.session.get("is_admin") == True

	@cached_property
	def auth_type(self):
		return self.session.get('auth_type')

	@property
	def email(self):
		return self.session.get(self._user_email_field)

	@property
	def subscriptions(self):
		return self.session.subscriptions
